@@ -123,7 +123,7 @@ StdCacheDelete(void *ptr)
     she = GetStdHashEntry(context);
 
     if (!she)
-        elog(ERROR, "StdCacheDelete: Trying to delete non-existant hash entry object with MemoryContext key (%p)", (void *)context);
+        elog(ERROR, "StdCacheDelete: Trying to delete non-existent hash entry object with MemoryContext key (%p)", (void *)context);
 
     DBG("deleting std object (%p) with MemoryContext key (%p)", she->std, context);
 
