@@ -30,8 +30,7 @@
 #include "liblwgeom_internal.h"  /* For FP comparators. */
 #include "lwgeom_pg.h"
 #include "math.h"
-#include "lwgeom_rtree.h"
-#include "lwgeom_functions_analytic.h"
+#include "lwgeom_itree.h"
 
 #include "access/htup_details.h"
 
@@ -47,11 +46,6 @@ Datum ST_IsPolygonCCW(PG_FUNCTION_ARGS);
 Datum ST_IsPolygonCW(PG_FUNCTION_ARGS);
 
 
-static double determineSide(const POINT2D *seg1, const POINT2D *seg2, const POINT2D *point);
-static int isOnSegment(const POINT2D *seg1, const POINT2D *seg2, const POINT2D *point);
-static int point_in_ring(POINTARRAY *pts, const POINT2D *point);
-static int point_in_ring_rtree(RTREE_NODE *root, const POINT2D *point);
-
 /***********************************************************************
  * Simple Douglas-Peucker line simplification.
  * No checks are done to avoid introduction of self-intersections.
@@ -657,447 +651,6 @@ Datum LWGEOM_line_substring(PG_FUNCTION_ARGS)
 
 }
 
-/*******************************************************************************
- * The following is based on the "Fast Winding Number Inclusion of a Point
- * in a Polygon" algorithm by Dan Sunday.
- * http://softsurfer.com/Archive/algorithm_0103/algorithm_0103.htm#Winding%20Number
- ******************************************************************************/
-
-/*
- * returns: >0 for a point to the left of the segment,
- *          <0 for a point to the right of the segment,
- *          0 for a point on the segment
- */
-static double determineSide(const POINT2D *seg1, const POINT2D *seg2, const POINT2D *point)
-{
-	return ((seg2->x-seg1->x)*(point->y-seg1->y)-(point->x-seg1->x)*(seg2->y-seg1->y));
-}
-
-/*
- * This function doesn't test that the point falls on the line defined by
- * the two points.  It assumes that that has already been determined
- * by having determineSide return within the tolerance.  It simply checks
- * that if the point is on the line, it is within the endpoints.
- *
- * returns: 1 if the point is not outside the bounds of the segment
- *          0 if it is
- */
-static int isOnSegment(const POINT2D *seg1, const POINT2D *seg2, const POINT2D *point)
-{
-	double maxX;
-	double maxY;
-	double minX;
-	double minY;
-
-	if (seg1->x > seg2->x)
-	{
-		maxX = seg1->x;
-		minX = seg2->x;
-	}
-	else
-	{
-		maxX = seg2->x;
-		minX = seg1->x;
-	}
-	if (seg1->y > seg2->y)
-	{
-		maxY = seg1->y;
-		minY = seg2->y;
-	}
-	else
-	{
-		maxY = seg2->y;
-		minY = seg1->y;
-	}
-
-	POSTGIS_DEBUGF(3, "maxX minX/maxY minY: %.8f %.8f/%.8f %.8f", maxX, minX, maxY, minY);
-
-	if (maxX < point->x || minX > point->x)
-	{
-		POSTGIS_DEBUGF(3, "X value %.8f falls outside the range %.8f-%.8f", point->x, minX, maxX);
-
-		return 0;
-	}
-	else if (maxY < point->y || minY > point->y)
-	{
-		POSTGIS_DEBUGF(3, "Y value %.8f falls outside the range %.8f-%.8f", point->y, minY, maxY);
-
-		return 0;
-	}
-	return 1;
-}
-
-/*
- * return -1 iff point is outside ring pts
- * return 1 iff point is inside ring pts
- * return 0 iff point is on ring pts
- */
-static int point_in_ring_rtree(RTREE_NODE *root, const POINT2D *point)
-{
-	int wn = 0;
-	uint32_t i;
-	double side;
-	const POINT2D *seg1;
-	const POINT2D *seg2;
-	LWMLINE *lines;
-
-	POSTGIS_DEBUG(2, "point_in_ring called.");
-
-	lines = RTreeFindLineSegments(root, point->y);
-	if (!lines)
-		return -1;
-
-	for (i=0; i<lines->ngeoms; i++)
-	{
-		seg1 = getPoint2d_cp(lines->geoms[i]->points, 0);
-		seg2 = getPoint2d_cp(lines->geoms[i]->points, 1);
-
-		side = determineSide(seg1, seg2, point);
-
-		POSTGIS_DEBUGF(3, "segment: (%.8f, %.8f),(%.8f, %.8f)", seg1->x, seg1->y, seg2->x, seg2->y);
-		POSTGIS_DEBUGF(3, "side result: %.8f", side);
-		POSTGIS_DEBUGF(3, "counterclockwise wrap %d, clockwise wrap %d", FP_CONTAINS_BOTTOM(seg1->y, point->y, seg2->y), FP_CONTAINS_BOTTOM(seg2->y, point->y, seg1->y));
-
-		/* zero length segments are ignored. */
-		if (((seg2->x - seg1->x)*(seg2->x - seg1->x) + (seg2->y - seg1->y)*(seg2->y - seg1->y)) < 1e-12*1e-12)
-		{
-			POSTGIS_DEBUG(3, "segment is zero length... ignoring.");
-
-			continue;
-		}
-
-		/* a point on the boundary of a ring is not contained. */
-		/* WAS: if (fabs(side) < 1e-12), see #852 */
-		if (side == 0.0)
-		{
-			if (isOnSegment(seg1, seg2, point) == 1)
-			{
-				POSTGIS_DEBUGF(3, "point on ring boundary between points %d, %d", i, i+1);
-
-				return 0;
-			}
-		}
-
-		/*
-		 * If the point is to the left of the line, and it's rising,
-		 * then the line is to the right of the point and
-		 * circling counter-clockwise, so increment.
-		 */
-		if ((seg1->y <= point->y) && (point->y < seg2->y) && (side > 0))
-		{
-			POSTGIS_DEBUG(3, "incrementing winding number.");
-
-			++wn;
-		}
-		/*
-		 * If the point is to the right of the line, and it's falling,
-		 * then the line is to the right of the point and circling
-		 * clockwise, so decrement.
-		 */
-		else if ((seg2->y <= point->y) && (point->y < seg1->y) && (side < 0))
-		{
-			POSTGIS_DEBUG(3, "decrementing winding number.");
-
-			--wn;
-		}
-	}
-
-	POSTGIS_DEBUGF(3, "winding number %d", wn);
-
-	if (wn == 0)
-		return -1;
-	return 1;
-}
-
-
-/*
- * return -1 iff point is outside ring pts
- * return 1 iff point is inside ring pts
- * return 0 iff point is on ring pts
- */
-static int point_in_ring(POINTARRAY *pts, const POINT2D *point)
-{
-	int wn = 0;
-	uint32_t i;
-	double side;
-	const POINT2D* seg1;
-	const POINT2D* seg2;
-
-	POSTGIS_DEBUG(2, "point_in_ring called.");
-
-	seg2 = getPoint2d_cp(pts, 0);
-	for (i=0; i<pts->npoints-1; i++)
-	{
-		seg1 = seg2;
-		seg2 = getPoint2d_cp(pts, i+1);
-
-		side = determineSide(seg1, seg2, point);
-
-		POSTGIS_DEBUGF(3, "segment: (%.8f, %.8f),(%.8f, %.8f)", seg1->x, seg1->y, seg2->x, seg2->y);
-		POSTGIS_DEBUGF(3, "side result: %.8f", side);
-		POSTGIS_DEBUGF(3, "counterclockwise wrap %d, clockwise wrap %d", FP_CONTAINS_BOTTOM(seg1->y, point->y, seg2->y), FP_CONTAINS_BOTTOM(seg2->y, point->y, seg1->y));
-
-		/* zero length segments are ignored. */
-		if ((seg2->x == seg1->x) && (seg2->y == seg1->y))
-		{
-			POSTGIS_DEBUG(3, "segment is zero length... ignoring.");
-
-			continue;
-		}
-
-		/* a point on the boundary of a ring is not contained. */
-		/* WAS: if (fabs(side) < 1e-12), see #852 */
-		if (side == 0.0)
-		{
-			if (isOnSegment(seg1, seg2, point) == 1)
-			{
-				POSTGIS_DEBUGF(3, "point on ring boundary between points %d, %d", i, i+1);
-
-				return 0;
-			}
-		}
-
-		/*
-		 * If the point is to the left of the line, and it's rising,
-		 * then the line is to the right of the point and
-		 * circling counter-clockwise, so increment.
-		 */
-		if ((seg1->y <= point->y) && (point->y < seg2->y) && (side > 0))
-		{
-			POSTGIS_DEBUG(3, "incrementing winding number.");
-
-			++wn;
-		}
-		/*
-		 * If the point is to the right of the line, and it's falling,
-		 * then the line is to the right of the point and circling
-		 * clockwise, so decrement.
-		 */
-		else if ((seg2->y <= point->y) && (point->y < seg1->y) && (side < 0))
-		{
-			POSTGIS_DEBUG(3, "decrementing winding number.");
-
-			--wn;
-		}
-	}
-
-	POSTGIS_DEBUGF(3, "winding number %d", wn);
-
-	if (wn == 0)
-		return -1;
-	return 1;
-}
-
-/*
- * return 0 iff point outside polygon or on boundary
- * return 1 iff point inside polygon
- */
-int point_in_polygon_rtree(RTREE_NODE **root, int ringCount, LWPOINT *point)
-{
-	int i;
-	POINT2D pt;
-
-	POSTGIS_DEBUGF(2, "point_in_polygon called for %p %d %p.", root, ringCount, point);
-
-	getPoint2d_p(point->point, 0, &pt);
-	/* assume bbox short-circuit has already been attempted */
-
-	if (point_in_ring_rtree(root[0], &pt) != 1)
-	{
-		POSTGIS_DEBUG(3, "point_in_polygon_rtree: outside exterior ring.");
-
-		return 0;
-	}
-
-	for (i=1; i<ringCount; i++)
-	{
-		if (point_in_ring_rtree(root[i], &pt) != -1)
-		{
-			POSTGIS_DEBUGF(3, "point_in_polygon_rtree: within hole %d.", i);
-
-			return 0;
-		}
-	}
-	return 1;
-}
-
-/*
- * return -1 if point outside polygon
- * return 0 if point on boundary
- * return 1 if point inside polygon
- *
- * Expected **root order is each exterior ring followed by its holes, eg. EIIEIIEI
- */
-int point_in_multipolygon_rtree(RTREE_NODE **root, int polyCount, int *ringCounts, LWPOINT *point)
-{
-	int i, p, r, in_ring;
-	POINT2D pt;
-	int result = -1;
-
-	POSTGIS_DEBUGF(2, "point_in_multipolygon_rtree called for %p %d %p.", root, polyCount, point);
-
-	getPoint2d_p(point->point, 0, &pt);
-	/* assume bbox short-circuit has already been attempted */
-
-        i = 0; /* the current index into the root array */
-
-	/* is the point inside any of the sub-polygons? */
-	for ( p = 0; p < polyCount; p++ )
-	{
-		in_ring = point_in_ring_rtree(root[i], &pt);
-		POSTGIS_DEBUGF(4, "point_in_multipolygon_rtree: exterior ring (%d), point_in_ring returned %d", p, in_ring);
-		if ( in_ring == -1 ) /* outside the exterior ring */
-		{
-			POSTGIS_DEBUG(3, "point_in_multipolygon_rtree: outside exterior ring.");
-		}
-         	else if ( in_ring == 0 ) /* on the boundary */
-		{
-			POSTGIS_DEBUGF(3, "point_in_multipolygon_rtree: on edge of exterior ring %d", p);
-                        return 0;
-		} else {
-                	result = in_ring;
-
-	                for(r=1; r<ringCounts[p]; r++)
-     	                {
-                        	in_ring = point_in_ring_rtree(root[i+r], &pt);
-		        	POSTGIS_DEBUGF(4, "point_in_multipolygon_rtree: interior ring (%d), point_in_ring returned %d", r, in_ring);
-                        	if (in_ring == 1) /* inside a hole => outside the polygon */
-                        	{
-                                	POSTGIS_DEBUGF(3, "point_in_multipolygon_rtree: within hole %d of exterior ring %d", r, p);
-                                	result = -1;
-                                	break;
-                        	}
-                        	if (in_ring == 0) /* on the edge of a hole */
-                        	{
-			        	POSTGIS_DEBUGF(3, "point_in_multipolygon_rtree: on edge of hole %d of exterior ring %d", r, p);
-                                	return 0;
-		        	}
-                	}
-                	/* if we have a positive result, we can short-circuit and return it */
-                	if ( result != -1)
-                	{
-                        	return result;
-                	}
-                }
-                /* increment the index by the total number of rings in the sub-poly */
-                /* we do this here in case we short-cutted out of the poly before looking at all the rings */
-                i += ringCounts[p];
-	}
-
-	return result; /* -1 = outside, 0 = boundary, 1 = inside */
-
-}
-
-/*
- * return -1 iff point outside polygon
- * return 0 iff point on boundary
- * return 1 iff point inside polygon
- */
-int point_in_polygon(LWPOLY *polygon, LWPOINT *point)
-{
-	uint32_t i;
-	int result, in_ring;
-	POINT2D pt;
-
-	POSTGIS_DEBUG(2, "point_in_polygon called.");
-
-	getPoint2d_p(point->point, 0, &pt);
-	/* assume bbox short-circuit has already been attempted */
-
-	/* everything is outside of an empty polygon */
-	if ( polygon->nrings == 0 ) return -1;
-
-	in_ring = point_in_ring(polygon->rings[0], &pt);
-	if ( in_ring == -1) /* outside the exterior ring */
-	{
-		POSTGIS_DEBUG(3, "point_in_polygon: outside exterior ring.");
-		return -1;
-	}
-	result = in_ring;
-
-	for (i=1; i<polygon->nrings; i++)
-	{
-		in_ring = point_in_ring(polygon->rings[i], &pt);
-		if (in_ring == 1) /* inside a hole => outside the polygon */
-		{
-			POSTGIS_DEBUGF(3, "point_in_polygon: within hole %d.", i);
-			return -1;
-		}
-		if (in_ring == 0) /* on the edge of a hole */
-		{
-			POSTGIS_DEBUGF(3, "point_in_polygon: on edge of hole %d.", i);
-			return 0;
-		}
-	}
-	return result; /* -1 = outside, 0 = boundary, 1 = inside */
-}
-
-/*
- * return -1 iff point outside multipolygon
- * return 0 iff point on multipolygon boundary
- * return 1 iff point inside multipolygon
- */
-int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *point)
-{
-	uint32_t i, j;
-	int result, in_ring;
-	POINT2D pt;
-
-	POSTGIS_DEBUG(2, "point_in_polygon called.");
-
-	getPoint2d_p(point->point, 0, &pt);
-	/* assume bbox short-circuit has already been attempted */
-
-	result = -1;
-
-	for (j = 0; j < mpolygon->ngeoms; j++ )
-	{
-
-		LWPOLY *polygon = mpolygon->geoms[j];
-
-		/* everything is outside of an empty polygon */
-		if ( polygon->nrings == 0 ) continue;
-
-		in_ring = point_in_ring(polygon->rings[0], &pt);
-		if ( in_ring == -1) /* outside the exterior ring */
-		{
-			POSTGIS_DEBUG(3, "point_in_polygon: outside exterior ring.");
-			continue;
-		}
-		if ( in_ring == 0 )
-		{
-			return 0;
-		}
-
-		result = in_ring;
-
-		for (i=1; i<polygon->nrings; i++)
-		{
-			in_ring = point_in_ring(polygon->rings[i], &pt);
-			if (in_ring == 1) /* inside a hole => outside the polygon */
-			{
-				POSTGIS_DEBUGF(3, "point_in_polygon: within hole %d.", i);
-				result = -1;
-				break;
-			}
-			if (in_ring == 0) /* on the edge of a hole */
-			{
-				POSTGIS_DEBUGF(3, "point_in_polygon: on edge of hole %d.", i);
-				return 0;
-			}
-		}
-		if ( result != -1)
-		{
-			return result;
-		}
-	}
-	return result;
-}
-
-
-/*******************************************************************************
- * End of "Fast Winding Number Inclusion of a Point in a Polygon" derivative.
- ******************************************************************************/
 
 /**********************************************************************
  *
@@ -1125,7 +678,7 @@ Datum ST_MinimumBoundingRadius(PG_FUNCTION_ARGS)
 
 	geom = PG_GETARG_GSERIALIZED_P(0);
 
-    /* Empty geometry?  Return POINT EMPTY with zero radius */
+	/* Empty geometry?  Return POINT EMPTY with zero radius */
 	if (gserialized_is_empty(geom))
 	{
 		lwcenter = (LWGEOM*) lwpoint_construct_empty(gserialized_get_srid(geom), LW_FALSE, LW_FALSE);
