@@ -20,62 +20,9 @@
 -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 
--- Helper function to drop functions when they match the full signature
--- Requires schema, name and __exact arguments__ as extracted from pg_catalog.pg_get_function_arguments
--- You can extract the old function arguments using a query like:
--- SELECT  p.oid as oid,
---                 n.nspname as schema,
---                 p.proname as name,
---                 pg_catalog.pg_get_function_arguments(p.oid) as arguments
---         FROM pg_catalog.pg_proc p
---         LEFT JOIN pg_catalog.pg_namespace n ON n.oid = p.pronamespace
---         WHERE
---                 pg_catalog.LOWER(n.nspname) = pg_catalog.LOWER('public') AND
---                 pg_catalog.LOWER(p.proname) = pg_catalog.LOWER('ST_AsGeoJson')
---         ORDER BY 1, 2, 3, 4;
-CREATE OR REPLACE FUNCTION _postgis_drop_function_if_needed(
-	function_name pg_catalog.text,
-	function_arguments pg_catalog.text) RETURNS void AS $$
-DECLARE
-	sql_drop pg_catalog.text;
-	postgis_namespace pg_catalog.OID;
-	matching_function REGPROCEDURE;
-BEGIN
-
-	-- Fetch install namespace for PostGIS
-	SELECT n.oid
-	FROM pg_catalog.pg_proc p
-	JOIN pg_catalog.pg_namespace n ON p.pronamespace = n.oid
-	WHERE proname = 'postgis_full_version'
-	INTO postgis_namespace;
-
-	-- Find a function matching the given signature
-	SELECT oid
-	FROM pg_catalog.pg_proc p
-	WHERE pronamespace = postgis_namespace
-	AND pg_catalog.LOWER(p.proname) = pg_catalog.LOWER(function_name)
-	AND pg_catalog.pg_function_is_visible(p.oid)
-	AND pg_catalog.LOWER(pg_catalog.pg_get_function_identity_arguments(p.oid)) ~ pg_catalog.LOWER(function_arguments)
-	INTO matching_function;
-
-	IF matching_function IS NOT NULL THEN
-		sql_drop := 'DROP FUNCTION ' || matching_function;
-		RAISE DEBUG 'SQL query: %', sql_drop;
-		BEGIN
-			EXECUTE sql_drop;
-		EXCEPTION
-			WHEN OTHERS THEN
-				RAISE EXCEPTION 'Could not drop function %. You might need to drop dependant objects. Postgres error: %', function_name, SQLERRM;
-		END;
-	END IF;
-
-END;
-$$ LANGUAGE plpgsql;
-
-
 -- FUNCTION AddGeometryColumn signature dropped
 -- (catalog_name character varying, schema_name character varying, table_name character varying, column_name character varying, new_srid integer, new_type character varying, new_dim integer, use_typmod boolean)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'AddGeometryColumn',
 	'catalog_name character varying, schema_name character varying, table_name character varying, column_name character varying, new_srid integer, new_type character varying, new_dim integer, use_typmod boolean'
@@ -83,7 +30,7 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION ST_AsX3D was changed to add versioning for 2.0
 -- (geom geometry, prec integer, options integer)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'ST_AsX3D',
 	'geom geometry, prec integer, options integer'
@@ -93,7 +40,7 @@ SELECT _postgis_drop_function_if_needed
 -- It changed the paramenter `new_srid` to `new_srid_in`
 -- (catalogn_name character varying, schema_name character varying, table_name character varying, column_name character varying, new_srid integer)
 -- Dropping it conditionally since the same signature still exists.
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'UpdateGeometrySRID',
 	'catalogn_name character varying, schema_name character varying, table_name character varying, column_name character varying, new_srid integer'
@@ -108,7 +55,7 @@ SELECT _postgis_drop_function_if_needed
 DO  language 'plpgsql' $$
 BEGIN
 	-- fix geometry ops --
-	IF EXISTS(SELECT oprname from pg_operator where oprname = '&&' AND oprrest::pg_catalog.text = 'geometry_gist_sel_2d') THEN
+	IF EXISTS(SELECT oprname from pg_operator where oprname = '&&' AND oprrest::text = 'geometry_gist_sel_2d') THEN
 	--it is bound to old name, drop new, rename old to new, install will fix body of code
 		DROP FUNCTION IF EXISTS gserialized_gist_sel_2d(internal, oid, internal, int4) ;
 		ALTER FUNCTION geometry_gist_sel_2d(internal, oid, internal, int4) RENAME TO gserialized_gist_sel_2d;
@@ -137,7 +84,7 @@ $$ ;
 -- FUNCTION ST_AsLatLonText went from multiple signatures to a single one with defaults for 2.2.0
 DROP FUNCTION IF EXISTS ST_AsLatLonText(geometry); -- Does not conflict
 
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'ST_AsLatLonText',
 	'geometry, text'
@@ -145,7 +92,7 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION ST_LineCrossingDirection changed argument names in 3.0
 -- Was (geom1 geometry, geom2 geometry) and now (line1 geometry, line2 geometry)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'ST_LineCrossingDirection',
 	'geom1 geometry, geom2 geometry'
@@ -153,7 +100,7 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION _st_linecrossingdirection changed argument names in 3.0
 -- Was (geom1 geometry, geom2 geometry) and now (line1 geometry, line2 geometry)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'_ST_LineCrossingDirection',
 	'geom1 geometry, geom2 geometry'
@@ -161,7 +108,7 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION ST_AsGeoJson changed argument names
 -- (pretty_print => pretty_bool) in 3.0alpha4
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'ST_AsGeoJson',
 	$args$r record, geom_column text, maxdecimaldigits integer, pretty_print boolean$args$
@@ -169,7 +116,7 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION _st_orderingequals changed argument names in 3.0
 -- Was (GeometryA geometry, GeometryB geometry) and now (geom1 geometry, geom2 geometry)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'_st_orderingequals',
 	'GeometryA geometry, GeometryB geometry'
@@ -177,28 +124,15 @@ SELECT _postgis_drop_function_if_needed
 
 -- FUNCTION st_orderingequals changed argument names in 3.0
 -- Was (GeometryA geometry, GeometryB geometry) and now (geom1 geometry, geom2 geometry)
-SELECT _postgis_drop_function_if_needed
+SELECT _postgis_drop_function_by_identity
 	(
 	'st_orderingequals',
 	'GeometryA geometry, GeometryB geometry'
 	);
 
--- FUNCTION st_tileenvelope added a new default argument in 3.1
-SELECT _postgis_drop_function_if_needed
-    (
-    'st_tileenvelope',
-    'zoom integer, x integer, y integer, bounds geometry'
-    );
-
-
--- FUNCTION st_buffer changed to add defaults in 3.0
--- This signature was superseeded
-DROP FUNCTION IF EXISTS st_buffer(geometry, double precision); -- Does not conflict
-
--- FUNCTION ST_CurveToLine changed to add defaults in 2.5
--- These signatures were superseeded
-DROP FUNCTION IF EXISTS ST_CurveToLine(geometry, integer); -- Does not conflict
-DROP FUNCTION IF EXISTS ST_CurveToLine(geometry); -- Does not conflict
+-- This was renamed mid-cycle in PostGIS 3.4.0 development
+-- to ST_CoverageInvalidEdges
+DROP FUNCTION IF EXISTS ST_CoverageInvalidLocations(geometry, double precision);
 
 -- geometry_columns changed parameter types so we verify if it needs to be dropped
 -- We check the catalog to see if the view (geometry_columns) has a column
@@ -238,8 +172,3 @@ BEGIN
 		END IF;
 END;
 $$;
-
-
--- DROP auxiliar function (created above)
-DROP FUNCTION _postgis_drop_function_if_needed(text, text);
-
