@@ -33,6 +33,10 @@
 
 #include <math.h>
 
+#ifndef NAN
+# define NAN 0.0/0.0
+#endif
+
 /**
  * Construct a raster with given dimensions.
  *
@@ -720,7 +724,7 @@ rt_raster_get_geotransform_matrix(rt_raster raster,
  * Set raster's geotransform using 6-element array
  *
  * @param raster : the raster to set matrix of
- * @param gt : intput parameter, 6-element geotransform matrix
+ * @param gt : input parameter, 6-element geotransform matrix
  *
  */
 void
@@ -789,7 +793,7 @@ rt_raster_cell_to_geopoint(
 }
 
 /**
- * Convert an xw,yw map point to a xr,yr raster point
+ * Convert an xw,yw map point to a xr,yr cell coordinate
  *
  * @param raster : the raster to get info from
  * @param xw : X ordinate of the geographical point
@@ -807,8 +811,51 @@ rt_raster_geopoint_to_cell(
 	double *xr, double *yr,
 	double *igt
 ) {
-	double _igt[6] = {0};
 	double rnd = 0;
+	rt_errorstate err;
+
+	err = rt_raster_geopoint_to_rasterpoint(raster, xw, yw, xr, yr, igt);
+	if (err != ES_NONE)
+		return err;
+
+	rnd = ROUND(*xr, 0);
+	if (FLT_EQ(rnd, *xr))
+		*xr = rnd;
+	else
+		*xr = floor(*xr);
+
+	rnd = ROUND(*yr, 0);
+	if (FLT_EQ(rnd, *yr))
+		*yr = rnd;
+	else
+		*yr = floor(*yr);
+
+	RASTER_DEBUGF(4, "Corrected GDALApplyGeoTransform (g -> c) for (%f, %f) = (%f, %f)",
+		xw, yw, *xr, *yr);
+
+	return ES_NONE;
+}
+
+/**
+ * Convert an xw,yw map point to a xr,yr raster point
+ *
+ * @param raster : the raster to get info from
+ * @param xw : X ordinate of the geographical point
+ * @param yw : Y ordinate of the geographical point
+ * @param xr : output parameter, the x ordinate in raster space
+ * @param yr : output parameter, the y ordinate in raster space
+ * @param igt : input/output parameter, inverse geotransform matrix
+ *
+ * @return ES_NONE if success, ES_ERROR if error
+ */
+rt_errorstate
+rt_raster_geopoint_to_rasterpoint(
+	rt_raster raster,
+	double xw, double yw,
+	double *xr, double *yr,
+	double *igt
+) {
+	double _igt[6] = {0};
 
 	assert(NULL != raster);
 	assert(NULL != xr && NULL != yr);
@@ -835,24 +882,10 @@ rt_raster_geopoint_to_cell(
 	RASTER_DEBUGF(4, "GDALApplyGeoTransform (g -> c) for (%f, %f) = (%f, %f)",
 		xw, yw, *xr, *yr);
 
-	rnd = ROUND(*xr, 0);
-	if (FLT_EQ(rnd, *xr))
-		*xr = rnd;
-	else
-		*xr = floor(*xr);
-
-	rnd = ROUND(*yr, 0);
-	if (FLT_EQ(rnd, *yr))
-		*yr = rnd;
-	else
-		*yr = floor(*yr);
-
-	RASTER_DEBUGF(4, "Corrected GDALApplyGeoTransform (g -> c) for (%f, %f) = (%f, %f)",
-		xw, yw, *xr, *yr);
-
 	return ES_NONE;
 }
 
+
 /******************************************************************************
 * rt_raster_get_envelope()
 ******************************************************************************/
@@ -1573,6 +1606,100 @@ rt_raster_clone(rt_raster raster, uint8_t deep) {
 	return rtn;
 }
 
+/******************************************************************************
+* rt_raster_copy_to_geometry()
+******************************************************************************/
+
+rt_errorstate
+rt_raster_copy_to_geometry(
+	rt_raster raster,
+	uint32_t bandnum,
+	char dim,
+	rt_resample_type resample,
+	const LWGEOM *lwgeom_in,
+	LWGEOM **lwgeom_out
+	)
+{
+	int has_z = lwgeom_has_z(lwgeom_in);
+	int has_m = lwgeom_has_m(lwgeom_in);
+	LWGEOM *lwgeom;
+	LWPOINTITERATOR* it;
+	POINT4D p;
+	double igt[6] = {0};
+	rt_errorstate err;
+	rt_band band = NULL;
+	double nodatavalue = 0.0;
+
+	/* Get the band reference and read the nodatavalue */
+	band = rt_raster_get_band(raster, bandnum);
+	if (!band) {
+		rterror("unable to read requested band");
+		return ES_ERROR;
+	}
+	rt_band_get_nodata(band, &nodatavalue);
+
+	/* Fluff up geometry to have space for our new dimension */
+	if (dim == 'z') {
+		if (has_z)
+			lwgeom = lwgeom_clone(lwgeom_in);
+		else if (has_m)
+			lwgeom = lwgeom_force_4d(lwgeom_in, nodatavalue, nodatavalue);
+		else
+			lwgeom = lwgeom_force_3dz(lwgeom_in, nodatavalue);
+	}
+	else if (dim == 'm') {
+		if (has_m)
+			lwgeom = lwgeom_clone(lwgeom_in);
+		if (has_z)
+			lwgeom = lwgeom_force_4d(lwgeom_in, nodatavalue, nodatavalue);
+		else
+			lwgeom = lwgeom_force_3dm(lwgeom_in, nodatavalue);
+	}
+	else {
+		rterror("unknown value for dim");
+		return ES_ERROR;
+	}
+
+	/* Read every point in the geometry */
+	it = lwpointiterator_create_rw(lwgeom);
+	while (lwpointiterator_has_next(it))
+	{
+		int nodata;
+		double xr, yr, value;
+		lwpointiterator_peek(it, &p);
+
+		/* Convert X/Y world coordinates into raster coordinates */
+		err = rt_raster_geopoint_to_rasterpoint(raster, p.x, p.y, &xr, &yr, igt);
+		if (err != ES_NONE) continue;
+
+		/* Read the raster value for this point */
+		err = rt_band_get_pixel_resample(
+			band,
+			xr, yr,
+			resample,
+			&value, &nodata
+		);
+
+		if (err != ES_NONE) {
+			value = NAN;
+		}
+
+		/* Copy in the raster value */
+		if (dim == 'z')
+			p.z = value;
+		if (dim == 'm')
+			p.m = value;
+
+		lwpointiterator_modify_next(it, &p);
+	}
+	lwpointiterator_destroy(it);
+
+	if (lwgeom_out)
+		*lwgeom_out = lwgeom;
+	return ES_NONE;
+}
+
+
 /******************************************************************************
 * rt_raster_to_gdal()
 ******************************************************************************/
@@ -1655,7 +1782,7 @@ rt_raster_to_gdal(
 		rtn_drv,
 		"/vsimem/out.dat", /* should be fine assuming this is in a process */
 		src_ds,
-		FALSE, /* should copy be strictly equivelent? */
+		FALSE, /* should copy be strictly equivalent? */
 		options, /* format options */
 		NULL, /* progress function */
 		NULL /* progress data */
@@ -2697,129 +2824,29 @@ rt_raster_gdal_rasterize(
 		_dim[0] == 0 &&
 		_dim[1] == 0
 	) {
-		int result;
-		LWPOLY *epoly = NULL;
-		LWGEOM *lwgeom = NULL;
-		GEOSGeometry *egeom = NULL;
-		GEOSGeometry *geom = NULL;
-
-		RASTER_DEBUG(3, "Testing geometry is properly contained by extent");
-
-		/*
-			see if geometry is properly contained by extent
-			all parts of geometry lies within extent
-		*/
-
-		/* initialize GEOS */
-		initGEOS(rtinfo, lwgeom_geos_error);
-
-		/* convert envelope to geometry */
-		RASTER_DEBUG(4, "Converting envelope to geometry");
-		epoly = rt_util_envelope_to_lwpoly(extent);
-		if (epoly == NULL) {
-			rterror("rt_raster_gdal_rasterize: Could not create envelope's geometry to test if geometry is properly contained by extent");
-
-			OGR_G_DestroyGeometry(src_geom);
-			_rti_rasterize_arg_destroy(arg);
-			/* OGRCleanupAll(); */
-
-			return NULL;
-		}
-
-		egeom = (GEOSGeometry *) LWGEOM2GEOS(lwpoly_as_lwgeom(epoly), 0);
-		lwpoly_free(epoly);
-
-		/* convert WKB to geometry */
-		RASTER_DEBUG(4, "Converting WKB to geometry");
-		lwgeom = lwgeom_from_wkb(wkb, wkb_len, LW_PARSER_CHECK_NONE);
-		geom = (GEOSGeometry *) LWGEOM2GEOS(lwgeom, 0);
-		lwgeom_free(lwgeom);
-
-		result = GEOSRelatePattern(egeom, geom, "T**FF*FF*");
-		GEOSGeom_destroy(geom);
-		GEOSGeom_destroy(egeom);
-
-		if (result == 2) {
-			rterror("rt_raster_gdal_rasterize: Could not test if geometry is properly contained by extent for geometry within extent");
-
-			OGR_G_DestroyGeometry(src_geom);
-			_rti_rasterize_arg_destroy(arg);
-			/* OGRCleanupAll(); */
-
-			return NULL;
-		}
-
-		/* geometry NOT properly contained by extent */
-		if (!result) {
 
 #if POSTGIS_GDAL_VERSION > 18
 
-			/* check alignment flag: grid_xw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_yw) &&
-				FLT_NEQ(*grid_xw, extent.MinX)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on X-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on X-axis");
-				extent.MinX -= (_scale[0] / 2.);
-				extent.MaxX += (_scale[0] / 2.);
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on X-axis");
+		extent.MinX -= (_scale[0] / 2.);
+		extent.MaxX += (_scale[0] / 2.);
 
-			/* check alignment flag: grid_yw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_yw) &&
-				FLT_NEQ(*grid_yw, extent.MaxY)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on Y-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on Y-axis");
-				extent.MinY -= (_scale[1] / 2.);
-				extent.MaxY += (_scale[1] / 2.);
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on Y-axis");
+		extent.MinY -= (_scale[1] / 2.);
+		extent.MaxY += (_scale[1] / 2.);
 
 #else
 
-			/* check alignment flag: grid_xw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_yw) &&
-				FLT_NEQ(*grid_xw, extent.MinX)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on X-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on X-axis");
-				extent.MinX -= _scale[0];
-				extent.MaxX += _scale[0];
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on X-axis");
+		extent.MinX -= _scale[0];
+		extent.MaxX += _scale[0];
 
-
-			/* check alignment flag: grid_yw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_yw) &&
-				FLT_NEQ(*grid_yw, extent.MaxY)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on Y-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on Y-axis");
-				extent.MinY -= _scale[1];
-				extent.MaxY += _scale[1];
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on Y-axis");
+		extent.MinY -= _scale[1];
+		extent.MaxY += _scale[1];
 
 #endif
 
-		}
 
 		RASTER_DEBUGF(3, "Adjusted extent: %f, %f, %f, %f",
 			extent.MinX, extent.MinY, extent.MaxX, extent.MaxY);
@@ -2945,7 +2972,7 @@ rt_raster_gdal_rasterize(
 			double _w[2] = {0};
 
 			/* raster is already aligned */
-			if (FLT_EQ(*grid_xw, extent.UpperLeftX) && FLT_EQ(*grid_yw, extent.UpperLeftY)) {
+			if (DBL_EQ(*grid_xw, extent.UpperLeftX) && DBL_EQ(*grid_yw, extent.UpperLeftY)) {
 				RASTER_DEBUG(3, "Skipping raster alignment as it is already aligned to grid");
 				break;
 			}
@@ -3341,14 +3368,14 @@ rt_raster_gdal_rasterize(
 			rt_band_get_nodata(oldband, &nodataval);
 
 		/* allocate data */
-		data = rtalloc(rt_pixtype_size(arg->pixtype[i]) * _width * _height);
+		data = rtalloc((size_t)rt_pixtype_size(arg->pixtype[i]) * _width * _height);
 		if (data == NULL) {
 			rterror("rt_raster_gdal_rasterize: Could not allocate memory for band data");
 			_rti_rasterize_arg_destroy(arg);
 			rt_raster_destroy(rast);
 			return NULL;
 		}
-		memset(data, 0, rt_pixtype_size(arg->pixtype[i]) * _width * _height);
+		memset(data, 0, (size_t)rt_pixtype_size(arg->pixtype[i]) * _width * _height);
 
 		/* create new band of correct type */
 		band = rt_band_new_inline(
