@@ -1,7 +1,7 @@
 \set VERBOSITY terse
 set client_min_messages to ERROR;
 
-\i ../load_topology.sql
+\i :top_builddir/topology/test/load_topology.sql
 \i ../load_features.sql
 
 SELECT 'relation before', count(distinct topogeo_id)
